@@ -99,7 +99,6 @@ xmlNodePtr _msMetadataGetOnline(xmlNsPtr namespace, layerObj *layer, const char
   int status;
   char *url = NULL;
   char buffer[32];
-  char *epsg_str;
   const char *link_protocol = "unknown protocol";
 
   xmlNodePtr psNode = NULL;
@@ -230,7 +229,7 @@ xmlNodePtr _msMetadataGetDecimal(xmlNsPtr namespace, const char *name, double va
 /*      Create a gmd:name/gmd:* code list element pattern               */
 /************************************************************************/
 
-xmlNodePtr _msMetadataGetCodeList(xmlNsPtr namespace, char *parent_element, char *name, char *value) {
+xmlNodePtr _msMetadataGetCodeList(xmlNsPtr namespace, const char *parent_element, const char *name, const char *value) {
   char *codelist = NULL;
   xmlNodePtr psNode = NULL;
   xmlNodePtr psCodeNode = NULL;
@@ -310,7 +309,7 @@ xmlNodePtr _msMetadataGetGMLTimePeriod(char **temporal)
 static
 xmlNodePtr _msMetadataGetExtent(xmlNsPtr namespace, layerObj *layer, xmlNsPtr *ppsNsGco)
 {
-  int n;
+  int n = 0;
   int status;
   char *value = NULL;
   char **temporal = NULL;
