@@ -48,10 +48,6 @@ extern FILE *msyyin;
 
 extern int msyystate;
 
-static const unsigned char PNGsig[8] = {137, 80, 78, 71, 13, 10, 26, 10}; /* 89 50 4E 47 0D 0A 1A 0A hex */
-static const unsigned char JPEGsig[3] = {255, 216, 255}; /* FF D8 FF hex */
-
-
 void freeImageCache(struct imageCacheObj *ic)
 {
   if(ic) {
